"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc2) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc2 = __getOwnPropDesc(from, key)) || desc2.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var common_exports = {};
__export(common_exports, {
  extractFilters: () => extractFilters,
  extractFiltersColumn: () => extractFiltersColumn,
  extractOrderBy: () => extractOrderBy,
  extractRelationsParams: () => extractRelationsParams,
  extractSelectedColumns: () => extractSelectedColumns,
  extractSelectedColumnsFromNode: () => extractSelectedColumnsFromNode,
  extractSelectedColumnsSQLFormat: () => extractSelectedColumnsSQLFormat,
  generateTableTypes: () => generateTableTypes,
  innerOrder: () => innerOrder
});
module.exports = __toCommonJS(common_exports);
var import_drizzle_orm = require("drizzle-orm");
var import_graphql = require("graphql");
var import_graphql_parse_resolve_info = require("graphql-parse-resolve-info");
var import_case_ops = require("../case-ops");
var import_data_mappers = require("../data-mappers");
var import_type_converter = require("../type-converter");
const extractSelectedColumns = (info, queryName) => {
  const tableSelection = info.operation.selectionSet.selections.find(
    (e) => e.kind === import_graphql.Kind.FIELD && e.name.value === queryName
  );
  if (!tableSelection || !tableSelection.selectionSet)
    return {};
  const selectedColumns = [];
  for (const columnSelection of tableSelection.selectionSet.selections) {
    if (columnSelection.kind !== import_graphql.Kind.FIELD)
      continue;
    selectedColumns.push([columnSelection.name.value, true]);
  }
  return Object.fromEntries(selectedColumns);
};
const extractSelectedColumnsFromNode = (info, table) => {
  if (!info.selectionSet)
    return {};
  const tableColumns = (0, import_drizzle_orm.getTableColumns)(table);
  const selectedColumns = [];
  for (const columnSelection of info.selectionSet.selections) {
    if (columnSelection.kind !== import_graphql.Kind.FIELD || !tableColumns[columnSelection.name.value])
      continue;
    selectedColumns.push([columnSelection.name.value, true]);
  }
  return Object.fromEntries(selectedColumns);
};
const extractSelectedColumnsSQLFormat = (info, queryName, table) => {
  const tableSelection = info.operation.selectionSet.selections.find(
    (e) => e.kind === import_graphql.Kind.FIELD && e.name.value === queryName
  );
  const selectedColumns = [];
  if (!tableSelection || !tableSelection.selectionSet)
    throw new import_graphql.GraphQLError("Received empty column selection!");
  for (const columnSelection of tableSelection.selectionSet.selections) {
    if (columnSelection.kind !== import_graphql.Kind.FIELD)
      continue;
    selectedColumns.push([columnSelection.name.value, table[columnSelection.name.value]]);
  }
  return Object.fromEntries(selectedColumns);
};
const innerOrder = new import_graphql.GraphQLInputObjectType({
  name: "InnerOrder",
  fields: {
    direction: {
      type: new import_graphql.GraphQLNonNull(
        new import_graphql.GraphQLEnumType({
          name: "OrderDirection",
          description: "Order by direction",
          values: {
            asc: {
              value: "asc",
              description: "Ascending order"
            },
            desc: {
              value: "desc",
              description: "Descending order"
            }
          }
        })
      )
    },
    priority: { type: new import_graphql.GraphQLNonNull(import_graphql.GraphQLInt), description: "Priority of current field" }
  }
});
const generateColumnFilterValues = (column, tableName, columnName) => {
  const columnGraphQLType = (0, import_type_converter.drizzleColumnToGraphQLType)(column, true);
  const columnArr = new import_graphql.GraphQLList(new import_graphql.GraphQLNonNull(columnGraphQLType.type));
  const baseFields = {
    eq: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    ne: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    lt: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    lte: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    gt: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    gte: { type: columnGraphQLType.type, description: columnGraphQLType.description },
    like: { type: import_graphql.GraphQLString },
    notLike: { type: import_graphql.GraphQLString },
    ilike: { type: import_graphql.GraphQLString },
    notIlike: { type: import_graphql.GraphQLString },
    inArray: { type: columnArr, description: `Array<${columnGraphQLType.description}>` },
    notInArray: { type: columnArr, description: `Array<${columnGraphQLType.description}>` },
    isNull: { type: import_graphql.GraphQLBoolean },
    isNotNull: { type: import_graphql.GraphQLBoolean }
  };
  const type = new import_graphql.GraphQLInputObjectType({
    name: `${(0, import_case_ops.pascalize)(tableName)}${(0, import_case_ops.pascalize)(columnName)}Filters`,
    fields: {
      ...baseFields,
      OR: {
        type: new import_graphql.GraphQLList(
          new import_graphql.GraphQLNonNull(
            new import_graphql.GraphQLInputObjectType({
              name: `${(0, import_case_ops.pascalize)(tableName)}${(0, import_case_ops.pascalize)(columnName)}filtersOr`,
              fields: {
                ...baseFields
              }
            })
          )
        )
      }
    }
  });
  return type;
};
const filterMap = /* @__PURE__ */ new WeakMap();
const generateTableFilterValuesCached = (table, tableName) => {
  if (filterMap.has(table))
    return filterMap.get(table);
  const columns = (0, import_drizzle_orm.getTableColumns)(table);
  const columnEntries = Object.entries(columns);
  const remapped = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      {
        type: generateColumnFilterValues(columnDescription, tableName, columnName)
      }
    ])
  );
  filterMap.set(table, remapped);
  return remapped;
};
const fieldMap = /* @__PURE__ */ new WeakMap();
const generateTableSelectTypeFieldsCached = (table) => {
  if (fieldMap.has(table))
    return fieldMap.get(table);
  const columns = (0, import_drizzle_orm.getTableColumns)(table);
  const columnEntries = Object.entries(columns);
  const remapped = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      (0, import_type_converter.drizzleColumnToGraphQLType)(columnDescription)
    ])
  );
  fieldMap.set(table, remapped);
  return remapped;
};
const generateTableTypes = (tableName, table, withReturning, relations) => {
  const relationEntries = relations ? Object.entries(relations) : [];
  const relationFields = Object.fromEntries(
    relationEntries.map(([relName, relValue]) => {
      const relTableFields = Object.fromEntries(
        Object.entries((0, import_drizzle_orm.getTableColumns)(relValue.referencedTable)).map(([columnName, columnDescription]) => [
          columnName,
          (0, import_type_converter.drizzleColumnToGraphQLType)(columnDescription)
        ])
      );
      const type = new import_graphql.GraphQLObjectType({
        name: `${(0, import_case_ops.pascalize)(tableName)}${(0, import_case_ops.pascalize)(relName)}Relation`,
        fields: relTableFields
      });
      const relationFilterColumns = generateTableFilterValuesCached(relValue.referencedTable, relName);
      const where = new import_graphql.GraphQLInputObjectType({
        name: `${(0, import_case_ops.pascalize)(tableName)}${(0, import_case_ops.pascalize)(relName)}RelationFilters`,
        fields: {
          ...relationFilterColumns,
          OR: {
            type: new import_graphql.GraphQLList(
              new import_graphql.GraphQLNonNull(
                new import_graphql.GraphQLInputObjectType({
                  name: `${(0, import_case_ops.pascalize)(tableName)}${(0, import_case_ops.pascalize)(relName)}RelationFiltersOr`,
                  fields: relationFilterColumns
                })
              )
            )
          }
        }
      });
      if ((0, import_drizzle_orm.is)(relValue, import_drizzle_orm.One))
        return [
          relName,
          {
            type,
            args: {
              where: { type: where }
            }
          }
        ];
      const orderBy = new import_graphql.GraphQLInputObjectType({
        name: `${(0, import_case_ops.pascalize)(tableName)}${(0, import_case_ops.pascalize)(relName)}RelationOrder`,
        fields: Object.fromEntries(
          Object.entries(relValue.referencedTable).map(([columnName, columnDescription]) => [
            columnName,
            { type: innerOrder }
          ])
        )
      });
      return [
        relName,
        {
          type: new import_graphql.GraphQLNonNull(new import_graphql.GraphQLList(new import_graphql.GraphQLNonNull(type))),
          args: {
            where: { type: where },
            orderBy: { type: orderBy },
            offset: { type: import_graphql.GraphQLInt },
            limit: { type: import_graphql.GraphQLInt }
          }
        }
      ];
    })
  );
  const tableFields = generateTableSelectTypeFieldsCached(table);
  const columns = (0, import_drizzle_orm.getTableColumns)(table);
  const columnEntries = Object.entries(columns);
  const insertFields = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      (0, import_type_converter.drizzleColumnToGraphQLType)(columnDescription, false, true)
    ])
  );
  const updateFields = Object.fromEntries(
    columnEntries.map(([columnName, columnDescription]) => [
      columnName,
      (0, import_type_converter.drizzleColumnToGraphQLType)(columnDescription, true)
    ])
  );
  const insertInput = new import_graphql.GraphQLInputObjectType({
    name: `${(0, import_case_ops.pascalize)(tableName)}InsertInput`,
    fields: insertFields
  });
  const selectSingleOutput = new import_graphql.GraphQLObjectType({
    name: `${(0, import_case_ops.pascalize)(tableName)}SelectItem`,
    fields: { ...tableFields, ...relationFields }
  });
  const selectArrOutput = new import_graphql.GraphQLNonNull(new import_graphql.GraphQLList(new import_graphql.GraphQLNonNull(selectSingleOutput)));
  const singleTableItemOutput = withReturning ? new import_graphql.GraphQLObjectType({
    name: `${(0, import_case_ops.pascalize)(tableName)}Item`,
    fields: tableFields
  }) : void 0;
  const arrTableItemOutput = withReturning ? new import_graphql.GraphQLNonNull(new import_graphql.GraphQLList(new import_graphql.GraphQLNonNull(singleTableItemOutput))) : void 0;
  const updateInput = new import_graphql.GraphQLInputObjectType({
    name: `${(0, import_case_ops.pascalize)(tableName)}UpdateInput`,
    fields: updateFields
  });
  const tableOrder = new import_graphql.GraphQLInputObjectType({
    name: `${(0, import_case_ops.pascalize)(tableName)}OrderBy`,
    fields: Object.fromEntries(
      columnEntries.map(([columnName, columnDescription]) => [columnName, { type: innerOrder }])
    )
  });
  const filterColumns = generateTableFilterValuesCached(table, tableName);
  const tableFilters = new import_graphql.GraphQLInputObjectType({
    name: `${(0, import_case_ops.pascalize)(tableName)}Filters`,
    fields: {
      ...filterColumns,
      OR: {
        type: new import_graphql.GraphQLList(
          new import_graphql.GraphQLNonNull(
            new import_graphql.GraphQLInputObjectType({
              name: `${(0, import_case_ops.pascalize)(tableName)}FiltersOr`,
              fields: filterColumns
            })
          )
        )
      }
    }
  });
  const inputs = {
    insertInput,
    updateInput,
    tableOrder,
    tableFilters
  };
  const outputs = withReturning ? {
    selectSingleOutput,
    selectArrOutput,
    singleTableItemOutput,
    arrTableItemOutput
  } : {
    selectSingleOutput,
    selectArrOutput
  };
  return {
    inputs,
    outputs
  };
};
const extractOrderBy = (table, orderArgs) => {
  const res = [];
  for (const [column, config] of Object.entries(orderArgs).sort(
    (a, b) => (b[1]?.priority ?? 0) - (a[1]?.priority ?? 0)
  )) {
    if (!config)
      continue;
    const { direction } = config;
    res.push(direction === "asc" ? (0, import_drizzle_orm.asc)((0, import_drizzle_orm.getTableColumns)(table)[column]) : (0, import_drizzle_orm.desc)((0, import_drizzle_orm.getTableColumns)(table)[column]));
  }
  return res;
};
const extractFiltersColumn = (column, columnName, operators) => {
  if (!operators.OR?.length)
    delete operators.OR;
  const entries = Object.entries(operators);
  if (operators.OR) {
    if (entries.length > 1)
      throw new import_graphql.GraphQLError(`WHERE ${columnName}: Cannot specify both fields and 'OR' in column operators!`);
    const variants2 = [];
    for (const variant of operators.OR) {
      const extracted = extractFiltersColumn(column, columnName, variant);
      if (extracted)
        variants2.push(extracted);
    }
    return variants2.length ? variants2.length > 1 ? (0, import_drizzle_orm.or)(...variants2) : variants2[0] : void 0;
  }
  const variants = [];
  for (const [operatorName, operatorValue] of entries) {
    if (operatorValue === null || operatorValue === false)
      continue;
    let operator;
    switch (operatorName) {
      case "eq":
        operator = operator ?? import_drizzle_orm.eq;
      case "ne":
        operator = operator ?? import_drizzle_orm.ne;
      case "gt":
        operator = operator ?? import_drizzle_orm.gt;
      case "gte":
        operator = operator ?? import_drizzle_orm.gte;
      case "lt":
        operator = operator ?? import_drizzle_orm.lt;
      case "lte":
        operator = operator ?? import_drizzle_orm.lte;
        const singleValue = (0, import_data_mappers.remapFromGraphQLCore)(operatorValue, column, columnName);
        variants.push(operator(column, singleValue));
        break;
      case "like":
        operator = operator ?? import_drizzle_orm.like;
      case "notLike":
        operator = operator ?? import_drizzle_orm.notLike;
      case "ilike":
        operator = operator ?? import_drizzle_orm.ilike;
      case "notIlike":
        operator = operator ?? import_drizzle_orm.notIlike;
        variants.push(operator(column, operatorValue));
        break;
      case "inArray":
        operator = operator ?? import_drizzle_orm.inArray;
      case "notInArray":
        operator = operator ?? import_drizzle_orm.notInArray;
        if (!operatorValue.length)
          throw new import_graphql.GraphQLError(
            `WHERE ${columnName}: Unable to use operator ${operatorName} with an empty array!`
          );
        const arrayValue = operatorValue.map((val) => (0, import_data_mappers.remapFromGraphQLCore)(val, column, columnName));
        variants.push(operator(column, arrayValue));
        break;
      case "isNull":
        operator = operator ?? import_drizzle_orm.isNull;
      case "isNotNull":
        operator = operator ?? import_drizzle_orm.isNotNull;
        variants.push(operator(column));
    }
  }
  return variants.length ? variants.length > 1 ? (0, import_drizzle_orm.and)(...variants) : variants[0] : void 0;
};
const extractFilters = (table, tableName, filters) => {
  if (!filters.OR?.length)
    delete filters.OR;
  const entries = Object.entries(filters);
  if (!entries.length)
    return;
  if (filters.OR) {
    if (entries.length > 1)
      throw new import_graphql.GraphQLError(`WHERE ${tableName}: Cannot specify both fields and 'OR' in table filters!`);
    const variants2 = [];
    for (const variant of filters.OR) {
      const extracted = extractFilters(table, tableName, variant);
      if (extracted)
        variants2.push(extracted);
    }
    return variants2.length ? variants2.length > 1 ? (0, import_drizzle_orm.or)(...variants2) : variants2[0] : void 0;
  }
  const variants = [];
  for (const [columnName, operators] of entries) {
    if (operators === null)
      continue;
    const column = (0, import_drizzle_orm.getTableColumns)(table)[columnName];
    variants.push(extractFiltersColumn(column, columnName, operators));
  }
  return variants.length ? variants.length > 1 ? (0, import_drizzle_orm.and)(...variants) : variants[0] : void 0;
};
const extractRelationsParams = (relations, tableSelection, typeName, info) => {
  const fields = {};
  const parsedInfo = (0, import_graphql_parse_resolve_info.parseResolveInfo)(info, {
    deep: true
  });
  const baseField = parsedInfo ? Object.entries(parsedInfo.fieldsByTypeName).find(([key, value]) => key === typeName)?.[1] ?? {} : {};
  for (const [relName, relValue] of Object.entries(relations)) {
    if (!tableSelection.selectionSet)
      continue;
    const node = tableSelection.selectionSet.selections.find(
      (e) => e.kind === import_graphql.Kind.FIELD && e.name.value === relName
    );
    if (!node)
      continue;
    const refTable = relValue.referencedTable;
    const columns = extractSelectedColumnsFromNode(node, refTable);
    const relationArgs = Object.values(baseField).find((e) => e.name === relName)?.args;
    if (!relationArgs) {
      fields[relName] = {
        columns
      };
      continue;
    }
    const orderBy = relationArgs.orderBy ? extractOrderBy(refTable, relationArgs.orderBy) : void 0;
    const where = relationArgs.where ? extractFilters(refTable, relName, relationArgs?.where) : void 0;
    const offset = relationArgs.offset ?? void 0;
    const limit = relationArgs.limit ?? void 0;
    fields[relName] = {
      columns,
      orderBy,
      where,
      offset,
      limit
    };
  }
  return Object.keys(fields).length ? fields : void 0;
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  extractFilters,
  extractFiltersColumn,
  extractOrderBy,
  extractRelationsParams,
  extractSelectedColumns,
  extractSelectedColumnsFromNode,
  extractSelectedColumnsSQLFormat,
  generateTableTypes,
  innerOrder
});
//# sourceMappingURL=common.cjs.map