"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var data_mappers_exports = {};
__export(data_mappers_exports, {
  remapFromGraphQLArrayInput: () => remapFromGraphQLArrayInput,
  remapFromGraphQLCore: () => remapFromGraphQLCore,
  remapFromGraphQLSingleInput: () => remapFromGraphQLSingleInput,
  remapToGraphQLArrayOutput: () => remapToGraphQLArrayOutput,
  remapToGraphQLCore: () => remapToGraphQLCore,
  remapToGraphQLSingleOutput: () => remapToGraphQLSingleOutput
});
module.exports = __toCommonJS(data_mappers_exports);
var import_drizzle_orm = require("drizzle-orm");
var import_graphql = require("graphql");
const remapToGraphQLCore = (key, value, relations) => {
  if (value instanceof Date)
    return value.toISOString();
  if (value instanceof Buffer)
    return Array.from(value);
  if (typeof value === "bigint")
    return value.toString();
  if (Array.isArray(value)) {
    if (relations?.[key])
      return remapToGraphQLArrayOutput(value);
    return value.map((arrVal) => remapToGraphQLCore(key, arrVal));
  }
  if (typeof value === "object") {
    if (relations?.[key])
      return remapToGraphQLSingleOutput(value);
    return JSON.stringify(value);
  }
  return value;
};
const remapToGraphQLSingleOutput = (queryOutput, relations) => {
  for (const [key, value] of Object.entries(queryOutput)) {
    if (value === void 0 || value === null) {
      delete queryOutput[key];
    } else {
      queryOutput[key] = remapToGraphQLCore(key, value, relations);
    }
  }
  return queryOutput;
};
const remapToGraphQLArrayOutput = (queryOutput, relations) => {
  for (const entry of queryOutput)
    remapToGraphQLSingleOutput(entry, relations);
  return queryOutput;
};
const remapFromGraphQLCore = (value, column, columnName) => {
  switch (column.dataType) {
    case "date": {
      const formatted = new Date(value);
      if (Number.isNaN(formatted.getTime()))
        throw new import_graphql.GraphQLError(`Field '${columnName}' is not a valid date!`);
      return formatted;
    }
    case "buffer": {
      if (!Array.isArray(value)) {
        throw new import_graphql.GraphQLError(`Field '${columnName}' is not an array!`);
      }
      return Buffer.from(value);
    }
    case "json": {
      try {
        return JSON.parse(value);
      } catch (e) {
        throw new import_graphql.GraphQLError(
          `Invalid JSON in field '${columnName}':
${e instanceof Error ? e.message : "Unknown error"}`
        );
      }
    }
    case "array": {
      if (!Array.isArray(value)) {
        throw new import_graphql.GraphQLError(`Field '${columnName}' is not an array!`);
      }
      return value;
    }
    case "bigint": {
      try {
        return BigInt(value);
      } catch (error) {
        throw new import_graphql.GraphQLError(`Field '${columnName}' is not a BigInt!`);
      }
    }
    default: {
      return value;
    }
  }
};
const remapFromGraphQLSingleInput = (queryInput, table) => {
  for (const [key, value] of Object.entries(queryInput)) {
    if (value === void 0) {
      delete queryInput[key];
    } else {
      const column = (0, import_drizzle_orm.getTableColumns)(table)[key];
      if (!column)
        throw new import_graphql.GraphQLError(`Unknown column: ${key}`);
      if (value === null && column.notNull) {
        delete queryInput[key];
        continue;
      }
      queryInput[key] = remapFromGraphQLCore(value, column, key);
    }
  }
  return queryInput;
};
const remapFromGraphQLArrayInput = (queryInput, table) => {
  for (const entry of queryInput)
    remapFromGraphQLSingleInput(entry, table);
  return queryInput;
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  remapFromGraphQLArrayInput,
  remapFromGraphQLCore,
  remapFromGraphQLSingleInput,
  remapToGraphQLArrayOutput,
  remapToGraphQLCore,
  remapToGraphQLSingleOutput
});
//# sourceMappingURL=index.cjs.map